package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.TuanYouConfig;
import com.hai.entity.*;
import com.hai.model.HighMerchantStoreModel;
import com.hai.model.HighUserModel;
import com.hai.model.ResponseData;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/26 23:08
 */
@Controller
@RequestMapping(value = "/highOrder")
@Api(value = "订单接口")
public class HighOrderController {
    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private HighCouponService highCouponService;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private HighCouponCodeService highCouponCodeService;

    @Resource
    private HighDiscountUserRelService highDiscountUserRelService;

    @Resource
    private HighDiscountCouponRelService highDiscountCouponRelService;

    @Resource
    private HighUserService highUserService;

    @RequestMapping(value="/addOrder",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加订单")
    public ResponseData addOrder(@RequestBody HighOrder highOrder, HttpServletRequest request) {
        try {
            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            if (highOrder.getHighChildOrderList() == null
                || highOrder.getHighChildOrderList().size() == 0
            ) {
                log.error("HighOrderController --> addOrder() error!", "参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            HighDiscountUserRel highDiscountUserRel = null;
            if (highOrder.getMemDiscountId() != null) {
                // 卡优惠券信息
                highDiscountUserRel = highDiscountUserRelService.getRelById(highOrder.getMemDiscountId());
                if (highDiscountUserRel == null || highDiscountUserRel.getStatus() != 1) {
                    log.error("HighOrderController --> addOrder() error!", "优惠券状态错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "优惠券状态错误");
                }
            }

            BigDecimal totalPrice = new BigDecimal("0");
            for (HighChildOrder childOrder : highOrder.getHighChildOrderList()) {
                if (childOrder.getGoodsType() == null || childOrder.getGoodsId() == null || childOrder.getSaleCount() == null) {
                    log.error("HighOrderController --> addOrder() error!", "参数错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                }

                // 商品类型   1:卡卷  2:金币充值  3:团油【加油站】
                if (childOrder.getGoodsType() == 1) {
                    if (childOrder.getSaleCount() != 1) {
                        log.error("HighOrderController --> addOrder() error!", "卡卷只能购买一张");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "卡卷只能购买一张");
                    }
                    // 查询卡卷信息
                    HighCoupon coupon = highCouponService.getCouponById(childOrder.getGoodsId());
                    if (coupon == null) {
                        log.error("HighOrderController --> addOrder() error!", "未找到卡卷信息");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_COUPON, "");
                    }
                    // 支付类型：1.微信支付 2.金币支付
                    if(coupon.getPayType() == 2) {
                        highOrder.setPayModel(1);  // 支付模式：1 金币，2 第三方平台，3 混合
                        highOrder.setPayType(3);   // 支付方式： 1：支付宝  2：微信  3：金币
                    }
                    // 是否达到限购数量
                    if (highCouponService.userBuyLimitNumber(userInfoModel.getHighUser().getId(), coupon.getId()) == true) {
                        log.error("HighOrderController --> addOrder() error!", "已达到限购数量");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, coupon.getCouponName() + ",已达到限购数量");
                    }
                    //  校验优惠券是否可用  如果使用优惠券就只能使用卡券的原价
                    if (highDiscountUserRel != null) {
                        List<HighDiscountCouponRel> discountCouponRelList = highDiscountCouponRelService.getRelByCoupon(coupon.getId());
                        if (discountCouponRelList.size() > 0) {
                            HighDiscountUserRel finalHighDiscountUserRel = highDiscountUserRel;
                            List<HighDiscountCouponRel> collect = discountCouponRelList.stream().filter(o -> o.getDiscountId().equals(finalHighDiscountUserRel.getDiscountId())).collect(Collectors.toList());
                            if (collect == null || collect.size() == 0) {
                                log.error("HighOrderController --> addOrder() error!", "无法使用优惠券");
                                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法使用优惠券");
                            }
                        }
                        childOrder.setGoodsPrice(coupon.getSalesPrice());
                    } else {
                        childOrder.setGoodsPrice(coupon.getDiscountPrice());
                    }
                    if (highCouponCodeService.getStockCountByCoupon(coupon.getId()) <= 0) {
                        log.error("HighOrderController --> addOrder() error!", "卡卷库存数量不足");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COUPON_STOCK_INSUFFICIENT, "");
                    }
                    childOrder.setGoodsName(coupon.getCouponName());
                    childOrder.setGoodsImg(coupon.getCouponImg());
                    childOrder.setGoodsSpecName("默认");
                }

                if (childOrder.getGoodsType() == 2) {
                    if (childOrder.getGoodsPrice() == null) {
                        log.error("HighOrderController --> addOrder() error!", "请设置充值金额");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "请设置充值金额");
                    }
                    // 查询用户信息
                    HighUser user = highUserService.findByUserId(childOrder.getGoodsId());
                    if (user == null) {
                        log.error("HighOrderController --> addOrder() error!", "未找到用户");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "");
                    }

                    childOrder.setGoodsName(user.getName());
                    childOrder.setGoodsImg(user.getHeaderImg());
                    childOrder.setGoodsSpecName("默认");
                }

                if (childOrder.getGoodsType() == 3) {
                    if (childOrder.getGoodsPrice() == null || StringUtils.isBlank(childOrder.getGasGunNo()) || StringUtils.isBlank(childOrder.getGasOilNo())) {
                        log.error("HighOrderController --> addOrder() error!", "参数错误");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
                    }
                    // 查询门店
                    HighMerchantStoreModel store = highMerchantStoreService.getMerchantStoreById(childOrder.getId());
                    if (store == null) {
                        log.error("HighOrderController --> addOrder() error!", "未找到门店信息");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到门店信息");
                    }

                    // 获取当前加油价格
                    JSONObject jsonObject = TuanYouConfig.queryCompanyPriceDetail(store.getStoreKey(), childOrder.getGasOilNo());
                    if (jsonObject == null || !jsonObject.getString("code").equals("200")) {
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未获取到加油价格");
                    }
                    if(jsonObject.getJSONArray("result").size() == 0) {
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未获取到加油价格");
                    }

                    JSONObject priceDetail = JSONObject.parseObject(JSONObject.toJSONString(jsonObject.getJSONArray("result").get(0)), JSONObject.class);
                    childOrder.setGoodsName(store.getStoreName());
                    childOrder.setGoodsImg(store.getStoreLogo());
                    childOrder.setGasPriceGun(priceDetail.getBigDecimal("priceGun"));
                    childOrder.setGasPriceVip(priceDetail.getBigDecimal("priceVip"));
                    childOrder.setGoodsPrice(new BigDecimal(childOrder.getGoodsPrice().toString()).divide(childOrder.getGasPriceGun(),2,BigDecimal.ROUND_HALF_UP).multiply(childOrder.getGasPriceVip()).setScale(2,BigDecimal.ROUND_HALF_UP));
                    childOrder.setGoodsSpecName("默认");
                    childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
                }

                childOrder.setMemId(userInfoModel.getHighUser().getId());
                childOrder.setTotalPrice(childOrder.getGoodsPrice().multiply(new BigDecimal(childOrder.getSaleCount().toString())));
                childOrder.setGiveawayType(false);  // 是否是赠品  0：否   1：是
                childOrder.setChildOrdeStatus(1);   // 1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消
                childOrder.setPraiseStatus(0);

                // 累计订单价格
                totalPrice = totalPrice.add(childOrder.getTotalPrice());
            }

            highOrder.setTotalPrice(totalPrice);
            highOrder.setPayPrice(highOrder.getTotalPrice());

            // 是否使用了优惠券
            if (highDiscountUserRel != null) {
                highOrder.setMemDiscountName(highDiscountUserRel.getHighDiscount().getDiscountName());

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 1) {
                    // 如果订单总额 小于 满减价格
                    if (highOrder.getTotalPrice().compareTo(highDiscountUserRel.getHighDiscount().getDiscountCondition()) > 1) {
                        log.error("HighOrderController --> addOrder() error!", "订单未达到满减额度");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到"+highDiscountUserRel.getHighDiscount().getDiscountCondition()+"元,无法使用此优惠券");
                    }
                    // 订单总额 - 满减额度
                    BigDecimal payPrice = highOrder.getTotalPrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());

                    highOrder.setDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (payPrice.compareTo(new BigDecimal("0")) == -1) {
                        highOrder.setPayPrice(new BigDecimal("0"));
                    } else {
                        highOrder.setPayPrice(payPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 2) {
                    // 订单总额 - 满减额度
                    BigDecimal payPrice = highOrder.getTotalPrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                    highOrder.setDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());

                    // 如果总额小于0
                    if (payPrice.compareTo(new BigDecimal("0")) == -1) {
                        highOrder.setPayPrice(new BigDecimal("0"));
                    } else {
                        highOrder.setPayPrice(payPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 3) {
                    // 折扣除100
                    BigDecimal discountPrice = highDiscountUserRel.getHighDiscount().getDiscountPrice().divide(new BigDecimal("10"));
                    // 订单总额 * 折扣
                    BigDecimal payPrice = highOrder.getTotalPrice().multiply(discountPrice);
                    highOrder.setDeductionPrice(highOrder.getTotalPrice().subtract(payPrice));
                    highOrder.setPayPrice(payPrice);
                }
            }

            highOrder.setOrderNo("HF" + DateUtil.date2String(new Date(),"yyyyMMddHHmmss") + IDGenerator.nextId(5));
            highOrder.setMemId(userInfoModel.getHighUser().getId());
            highOrder.setMemName(userInfoModel.getHighUser().getName());
            highOrder.setMemPhone(userInfoModel.getHighUser().getPhone());
            highOrder.setCreateTime(new Date());
            highOrder.setOrderStatus(1);

            highOrderService.insertOrder(highOrder);

            return ResponseMsgUtil.success(highOrderService.getOrderById(highOrder.getId()));

        } catch (Exception e) {
            log.error("HighOrderController -> addOrder() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    public static void main(String[] args) {
    }

    @RequestMapping(value = "/getOrderById", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询订单详情")
    public ResponseData getOrderById(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {
            return ResponseMsgUtil.success(highOrderService.getOrderById(orderId));
        } catch (Exception e) {
            log.error("HighOrderController --> getOrderById() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

/*
    @RequestMapping(value = "/calculateOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "计算订单详情")
    public ResponseData calculateOrder(@RequestParam(name = "orderId", required = true) Long orderId,
                                       @RequestParam(name = "userDiscountId", required = false) Long userDiscountId) {
        try {

            HighOrder order = highOrderService.getOrderById(orderId);
            if (order != null && userDiscountId != null) {
                // 卡优惠券信息
                HighDiscountUserRel rel = highDiscountUserRelService.getRelById(userDiscountId);
                order.setMemDiscountId(userDiscountId);
                order.setMemDiscountName(rel.getHighDiscount().getDiscountName());

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 1) {
                    // 如果订单总额 小于 满减价格
                    if (order.getTotalPrice().compareTo(rel.getHighDiscount().getDiscountCondition()) > 1) {
                        log.error("HighOrderController --> addOrder() error!", "订单未达到满减额度");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到"+rel.getHighDiscount().getDiscountCondition()+"元,无法使用此优惠券");
                    }
                    // 订单总额 - 满减额度
                    BigDecimal totalPrice = order.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (totalPrice.compareTo(new BigDecimal("0")) == -1) {
                        order.setTotalPrice(new BigDecimal("0"));
                    } else {
                        order.setTotalPrice(totalPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 2) {
                    // 订单总额 - 满减额度
                    BigDecimal totalPrice = order.getTotalPrice().subtract(rel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (totalPrice.compareTo(new BigDecimal("0")) == -1) {
                        order.setTotalPrice(new BigDecimal("0"));
                    } else {
                        order.setTotalPrice(totalPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (rel.getHighDiscount().getDiscountType() == 3) {
                    // 折扣除100
                    BigDecimal discountPrice = rel.getHighDiscount().getDiscountPrice().divide(new BigDecimal("100"));
                    // 订单总额 * 折扣
                    BigDecimal totalPrice = order.getTotalPrice().multiply(discountPrice);
                    order.setTotalPrice(totalPrice);
                }
            }
            return ResponseMsgUtil.success(order);

        } catch (Exception e) {
            log.error("HighOrderController --> cancelOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
*/

    @RequestMapping(value = "/cancelOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "取消订单")
    public ResponseData cancelOrder(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {

            highOrderService.cancelOrder(orderId);

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighOrderController --> cancelOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getUserOrderList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取用户订单")
    public ResponseData getUserOrderList(
            @RequestParam(name = "status", required = false) Integer status,
            @RequestParam(name = "orderNo", required = false) String orderNo,
            @RequestParam(name = "pageNum", required = true) Integer pageNum,
            @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                         HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            Map<String,Object> map = new HashMap<>();
            map.put("memId", userInfoModel.getHighUser().getId());
            map.put("status", status);
            map.put("orderNo", orderNo);

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(highOrderService.getOrderList(map)));

        } catch (Exception e) {
            log.error("HighOrderController --> getUserOrderList() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
